# frozen_string_literal: true

class Reynard
  # Superclass for dynamic classes generated by the object builder.
  class Model
    def initialize(attributes)
      self.attributes = attributes
    end

    def attributes=(attributes)
      attributes.each do |name, value|
        instance_variable_set("@#{name}", value)
      end
    end

    # Until we can set accessors based on the schema
    def method_missing(attribute_name, *)
      instance_variable_get("@#{attribute_name}")
    rescue NameError
      raise NoMethodError, "undefined method `#{attribute_name}' for #{inspect}"
    end

    def respond_to_missing?(attribute_name, *)
      !instance_variable_get("@#{attribute_name}").nil?
    rescue NameError
      false
    end
  end
end
